#!/usr/bin/perl

#
# xwiki2html.pl
#
# Convert XWiki pages into plain html
#
# 2008-03-22 dpj@us.ibm.com: Created
#

use LWP;
use Crypt::SSLeay;

package xwiki2html;
use base "HTML::Parser";

# Script debugging
$DEBUG_INPUT = 0x1;
$DEBUG_PARSE = 0x2;
$DEBUG_LINKS = 0x4;
$debug = 0;
#$debug = $DEBUG_PARSE;

$CRLF="\r\n";

sub start {
  my ($self, $tag, $attr, $attrseq, $origtext) = @_;
  print "START($copyingDepth) TAG($tag) ORIGTEXT($origtext)$CRLF" if ($debug & $DEBUG_PARSE);
  if ($tag =~ /^div$/) {
    if ($copyingDepth > 0) {
      $copyingDepth++;
    } elsif ($origtext =~ /class="main-content"/) {
      $copyingDepth++;
    }
  }
  if ($copyingDepth > 0) {
    if ($origtext =~ /href=\"(.*)\"/) {
      my $href = $1;
      if ( $href =~ /^$urlBase(.*)$/) {
		$href = $1;
		my $found = 0;
		for $p (@pages) {
		  if ($p == $href) {
		    $found = 1;
		  }
		}
		if ($found) {
		  $href =~ s/\+/_/g;
		  my $file = $href . ".html";
		  $origtext =~ s/href=\"(.*)\"/href=\"$file\"/;
		  print "Resolved local href=$href$CRLF" if ($debug & $DEBUG_LINKS);
		} else {
		  print "Unresolved local href=$href$CRLF";
		}
      } else {
		print "Unresolved remote href=$href$CRLF";
      }
    }
    if ($origtext =~ /<p class=\"paragraph\"\/>/) {
    	$origtext = "<p class=\"paragraph\"\></p>";
    }
    if (($capturingTitle == 0) && ($origtext =~ /class=\"heading-1-1\"/)) {
      $capturingTitle = 1;
    }
    if ($origtext =~ /<h3/) {
	    if ($origtext =~ /class=\"heading-1\"/) {
      		$processingH1 = 1;
      		print OUT "<h1>";
    	} elsif ($origtext =~ /class=\"heading-1-1\"/) {
	        $processingH2 = 1;
      		print OUT "<h2>";
    	} elsif ($origtext =~ /class=\"heading-1-1-1\"/) {
	        $processingH3 = 1;
      		print OUT "<h3>";
      	} else {
      		print "Unrecognized header=$origtext$CRLF";
      	}
    } else {
   	  print OUT $origtext;
   	}
  }
}

sub end {
  my ($self, $tag, $origtext) = @_;
  print "END($copyingDepth)   TAG($tag) ORIGTEXT($origtext)$CRLF" if ($debug & $DEBUG_PARSE);
  if ($copyingDepth > 0) {
    if ($origtext =~ /<\/h3>/) {
	    if ($processingH1) {
      		$processingH1 = 0;
      		print OUT "<\/h1>";
    	} elsif ($processingH2) {
      		$processingH2 = 0;
      		print OUT "<\/h2>";
    	} elsif ($processingH3) {
      		$processingH3 = 0;
      		print OUT "<\/h3>";
      	} else {
      		print "Unrecognized header=$origtext$CRLF";
      	}
    } else {
   	  print OUT $origtext;
   	}
    if ($tag =~ /^div$/) {
      $copyingDepth--;
      print OUT "$CRLF";
    }
  }
}

sub text {
  my ($self, $text) = @_;
  print "TEXT($copyingDepth)($text)$CRLF" if ($debug & $DEBUG_PARSE);
  if ($copyingDepth > 0) {
    $text =~ s/\r//g;
    $text =~ s/&#123;/\{/g;
    $text =~ s/&#125;/\}/g;
    $text =~ s/&#42;/\*/g;
    $text =~ s/&#60;/&lt;/g;
    $text =~ s/&#62;/&gt;/g;
    $text =~ s/&#91;/\[/g;
    $text =~ s/&#93;/\]/g;
    $text =~ s/&#45;/\-/g;
    $text =~ s/&#34;/\"/g;
    $text =~ s/&#8230;/\.\.\./g;
    $text =~ s/&#110;/\\n/g;
    $text =~ s/\[</[$lt;/g;
    $text =~ s/>\]/&gt;]/g;
    $text =~ s/&#35;/\#/g;
    $text =~ s/&#48;/\\0/g;
    $text =~ s/&#67;/\\C/g;
    $text =~ s/&#68;/\\D/g;
    $text =~ s/&#87;/\\W/g;
    $text =~ s/&#110;/\\n/g;
    $text =~ s/&#115;/\\s/g;
    print OUT $text;
  }
  if ($capturingTitle == 1) {
    $title = $text;
    $capturingTitle = 2;
  }
}

sub comment {
  my ($self, $comment) = @_;
  print "COMMENT($copyingDepth)($comment)$CRLF" if ($debug & $DEBUG_PARSE);
  if ($copyingDepth > 0) {
    $comment =~ s/\r//g;
    print OUT $comment;
  }
}

sub declaration {
  my ($self, $decl) = @_;
  print "DECLARATION($copyingDepth)($decl)$CRLF" if ($debug & $DEBUG_PARSE);
  if ($copyingDepth > 0) {
    $decl =~ s/\r//g;
    print OUT $decl;
  }
}

sub generate {
  my ($self, $name) = @_;

  $copyingDepth = 0;

  $title = "UNKNOWN";
  $capturingTitle = 0;

  my $url = $urlHost . $urlBase . $name;
  my $response = $ua->get($url);

  my $file = $name;
  $file =~ s/\+/_/g;
  $file = $file . ".html";
  open(OUT, ">$file");

  print OUT "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">$CRLF";
  print OUT "<HTML>$CRLF";
  print OUT "<HEAD>$CRLF";
  print OUT "<META http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">$CRLF";
  print OUT "<LINK rel=\"stylesheet\" href=\"../swg_info_common.css\" type=\"text/css\">$CRLF";
  print OUT "</HEAD>$CRLF";
  print OUT "<BODY>$CRLF";
  $self->parse($response->as_string);
  print OUT "<p class=\"runningfooter\">&copy; Copyright IBM Corp. 2007, 2008 All Rights Reserved. US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.</p>";
  print OUT "</BODY>$CRLF";
  print OUT "</HTML>$CRLF";

  close(OUT);

  print TOC "	<topic label=\"$title\" href=\"$tocFileBase/$file\"></topic>$CRLF";
}

$ua = LWP::UserAgent->new;
$parser = new xwiki2html;

$urlHost = "https://autosupport.rchland.ibm.com";
$urlBase = "/xwiki/bin/view/LI4425Tut/";
$tocFileBase = "docs/WritingAnalyzersTutorial";

@pages = (
	  'WebHome',
	  'Initial+setup+to+write+and+test+WAS-related+analyzers',
	  'HelloWorld', 
	  'A+Simple+Analyzer+to+Examine+Data+Structures',
	  'Writing+and+Formatting+Reports',
	  'Printing+Groups+of+ObjectWrapper+Fields',
	  'Working+with+J2SE+Collections',
	  'A+Custom+Wrapper+Analyzer',
	  'Extracting+Values+from+Data+Structures',
	  'Invoking+Another+Analyzer',
	  'Generating+Observations',
	  'Invoking+DTFJ+Functions',
	  'Generating+Reports+with+a+Summary+and+a+Details+Section',
	  'Integrating+the+Analyzer+into+the+Tool+Menu',
      'SampleSource',
      'SampleOutput'	 
    );

open(TOC, ">toc.xml");
print TOC "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>$CRLF";
print TOC "<?NLS TYPE=\"org.eclipse.help.toc\"?>$CRLF";    
print TOC " $CRLF";
print TOC "<toc label=\"xwiki2html\">$CRLF";

for $page (@pages) {
  print "=== PROCESSING PAGE $page$CRLF";
  $parser->generate($page);
}

print TOC "</toc>$CRLF";
close(TOC);


